import json
import base64
import os

from flask import request, jsonify, g, current_app
from flask_restx import Resource, Namespace, reqparse
from sqlalchemy import select, insert, update, delete, func
from sqlalchemy.orm import Session
from werkzeug.datastructures import FileStorage

from app.defines import StatesCode, Module, OperationType
from app.modle.data import Template, CompanyData, Building, UnderlyingSystem
from app.utils.jwt_util import login_required
from app.utils.save_log import save_log
from app.utils.util import to_dict, cn_now
from config import Config

ns = Namespace('data', description='数据管理接口')

template_list = reqparse.RequestParser(bundle_errors=True)
template_list.add_argument(name='page_size', type=int, location='args', required=False, help='每页记录数量，默认:20')
template_list.add_argument(name='page', type=int, location='args', required=False, help='第几页')
template_list.add_argument(name='template_name', type=str, location='args', required=False, help='模版名称')
template_list.add_argument(name='template_format', type=str, location='args', required=False, help='模版格式')
template_list.add_argument(name='template_type', type=str, location='args', required=False,
                           help='模版类型 0:报表，1:报告')
template_list.add_argument(name='report_type', type=str, location='args', required=False, help='报告类型')

config = Config()


@ns.route('/template_list')
class TemplateConfigListApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='template_list', description='模版列表')
    @ns.expect(template_list)
    def get(self):
        """获取模版列表"""
        page_size = int(request.args.get('page_size', 20))
        page = int(request.args.get('page', 1))
        template_name = request.args.get('template_name')
        template_format = request.args.get('template_format')
        report_type = request.args.get('report_type')
        template_type = request.args.get('template_type')

        try:
            with Session(current_app.engine) as session:
                stmt = select(Template).where(Template.template_type == template_type)

                count = select(func.count(Template.id)).where(Template.template_type == template_type)
                count_results = session.execute(count).scalars().first()

                if template_name:
                    stmt = stmt.where(Template.name == template_name)

                if template_format:
                    stmt = stmt.where(Template.format == template_format)

                if report_type:
                    stmt = stmt.where(Template.report_type == report_type)

                stmt = stmt.offset(page_size * (page - 1)).limit(page_size)

                results = session.execute(stmt).scalars().all()

            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', total=count_results, data=to_dict(results))
        except Exception as e:
            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


template = reqparse.RequestParser(bundle_errors=True)
template.add_argument(name='template_id', type=int, location='args', required=False, help='模版id')

building_details = reqparse.RequestParser(bundle_errors=True)
building_details.add_argument(name='template_id', type=int, location='form', required=False, help='模版id')
building_details.add_argument(name='template_name', type=str, location='form', required=False, help='模版名称')
building_details.add_argument(name='template_format', type=str, location='form', required=False, help='模版格式')
building_details.add_argument(name='report_type', type=str, location='form', required=False, help='报告类型')
building_details.add_argument(name='introduction', type=str, location='form', required=False, help='模版简介')
building_details.add_argument(name='template_file', type=FileStorage, location='files', required=False, help='模版文件')
building_details.add_argument(name='template_type', type=int, location='files', required=False,
                              help='模版类型 0:报表，1:报告')


@ns.route('/template')
class TemplateConfigApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='get_template', description='获取模版详情')
    @ns.expect(template)
    def get(self):
        """获取模版详情"""
        template_id = request.args.get('template_id')

        if template_id is None:
            return jsonify(code=StatesCode.PARA_ERROR, message='模版id不能为空')

        try:
            with Session(current_app.engine) as session:
                stmt = select(Template).where(Template.id == template_id)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='获取成功', data=to_dict(results))
        except Exception as e:
            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='add_template', description='添加报表模版')
    @ns.expect(building_details)
    def post(self):
        """添加模版"""
        template_name = request.form.get('template_name')
        template_format = request.form.get('template_format')
        introduction = request.form.get('introduction')
        template_file = request.files.get('template_file')
        report_type = request.form.get('report_type')
        template_type = request.form.get('template_type')

        if template_name is None and template_type is None:
            return jsonify(code=StatesCode.PARA_ERROR, message='报表模版名和模版类型不能为空')

        if template_file:
            template_file.save(os.path.join(config.common.SAVE_FILE_PATH, template_file.filename))
            template_file_name = template_file.filename
        else:
            template_file_name = None

        try:
            with Session(current_app.engine) as session:
                # 判断是否存在
                stmt = select(Template).where(Template.name == template_name)
                result = session.execute(stmt).scalars().first()
                if result:
                    return jsonify(code=StatesCode.UNKNOWN_ERROR, message="模版已存在")

                # 添加
                stmt = insert(Template).values(
                    name=template_name,
                    format=template_format,
                    introduction=introduction,
                    report_type=report_type,
                    creator=g.user_name,
                    create_time=cn_now(),
                    update_time=cn_now(),
                    template=template_file_name,
                    template_type=template_type
                )
                session.execute(stmt)

                session.commit()

            save_log(request, Module.DATA, OperationType.ADD, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='添加成功')
        except Exception as e:
            save_log(request, Module.DATA, OperationType.ADD, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='update_templatet', description='修改模版信息')
    @ns.expect(building_details)
    def put(self):
        """修改模版"""
        template_id = request.form.get('template_id')
        template_name = request.form.get('template_name')
        template_format = request.form.get('template_format')
        introduction = request.form.get('introduction')
        template_file = request.files.get('template_file')
        report_type = request.form.get('report_type')
        template_type = request.form.get('template_type')

        if template_id is None or template_name is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="模版id和模版名不能为空")

        if template_file:
            template_file.save(os.path.join(config.common.SAVE_FILE_PATH, template_file.filename))
            template_file_name = template_file.filename
        else:
            template_file_name = None
        try:
            with Session(current_app.engine) as session:
                stmt = update(Template).where(Template.id == template_id).values(
                    name=template_name,
                    format=template_format,
                    introduction=introduction,
                    report_type=report_type,
                    creator=g.user_name,
                    create_time=cn_now(),
                    update_time=cn_now(),
                    template=template_file_name,
                    template_type=template_type
                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.DATA, OperationType.UPDATE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='修改成功')
        except Exception as e:
            save_log(request, Module.DATA, OperationType.UPDATE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='delete_template', description='删除模版')
    @ns.expect(building_details)
    def delete(self):
        """删除模版"""
        template_id = request.form.get('template_id')

        if template_id is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="模版id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = delete(Template).where(Template.id == template_id)
                session.execute(stmt)
                session.commit()

            save_log(request, Module.DATA, OperationType.DELETE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='删除成功')
        except Exception as e:
            save_log(request, Module.DATA, OperationType.DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


batch_delete_template = reqparse.RequestParser(bundle_errors=True)
batch_delete_template.add_argument(name='template_ids', type=str, required=True, location='form', help='模版id列表')


@ns.route('/batch_delete_template')
class BatchTemplateConfigApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='batch_delete_template', description='批量删除模版')
    @ns.expect(batch_delete_template)
    def delete(self):
        """批量删除模版"""
        template_ids = request.form.get('template_ids')

        if template_ids is not None:
            template_ids = json.loads(template_ids)
        else:
            return jsonify(code=StatesCode.PARA_ERROR, message='模版id不能为空')

        try:
            with Session(current_app.engine) as session:
                stmt = select(Template).where(Template.id.in_(template_ids))
                results = session.execute(stmt).scalars().all()

                for result in results:
                    session.delete(result)
                session.commit()

            save_log(request, Module.DATA, OperationType.BATCH_DELETE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='批量删除成功')
        except Exception as e:
            save_log(request, Module.DATA, OperationType.BATCH_DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


company_id = reqparse.RequestParser(bundle_errors=True)
company_id.add_argument(name='company_id', type=int, location='args', required=False, help='公司id')

company_data = reqparse.RequestParser(bundle_errors=True)
company_data.add_argument(name='company_id', type=int, location='form', required=False, help='公司id')
company_data.add_argument(name='management_unit', type=str, location='form', required=False, help='管理单位')
company_data.add_argument(name='custodian_unit', type=str, location='form', required=False, help='托管单位')
company_data.add_argument(name='introduction', type=str, location='form', required=False, help='楼宇简介')
company_data.add_argument(name='contact_person', type=str, location='form', required=False, help='联系人')
company_data.add_argument(name='contact_information', type=str, location='form', required=False, help='联系人方式')
company_data.add_argument(name='picture', type=FileStorage, location='files', required=False, help='公司图片')


@ns.route('/company')
class CompanyApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='get_company', description='获取公司信息')
    @ns.expect(company_id)
    def get(self):
        """获取公司基本信息"""
        company_id = request.args.get('company_id')

        if company_id is None:
            return jsonify(code=StatesCode.PARA_ERROR, message='公司id不能为空')

        try:
            with Session(current_app.engine) as session:
                stmt = select(CompanyData).where(CompanyData.id == company_id)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='获取成功', data=to_dict(results))

        except Exception as e:
            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='add_company', description='添加公司信息')
    @ns.expect(company_data)
    def post(self):
        """添加公司基本信息"""
        management_unit = request.form.get('management_unit')
        custodian_unit = request.form.get('custodian_unit')
        introduction = request.form.get('introduction')
        contact_person = request.form.get('contact_person')
        contact_information = request.form.get('contact_information')
        pictures = request.files.getlist('picture')

        if management_unit is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message='管理单位不能为空')

        pictures_base64 = []

        if pictures:
            for picture in pictures:
                ext = picture.filename.split('.')[-1]
                base64_data = b'data:image/%s;base64,' % ext.encode('utf-8') + base64.b64encode(picture.read())
                base64_data = base64_data.decode('utf-8')
                pictures_base64.append(base64_data)

        try:
            with Session(current_app.engine) as session:
                # 判断是否存在
                stmt = select(CompanyData).where(CompanyData.management_unit == management_unit)
                result = session.execute(stmt).scalars().first()
                if result:
                    return jsonify(code=StatesCode.UNKNOWN_ERROR, message="管理单位已存在")

                # 添加
                stmt = insert(CompanyData).values(
                    management_unit=management_unit,
                    custodian_unit=custodian_unit,
                    introduction=introduction,
                    contact_person=contact_person,
                    contact_information=contact_information,
                    picture=json.dumps(pictures_base64),
                )
                session.execute(stmt)

                session.commit()

            save_log(request, Module.DATA, OperationType.ADD, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='新增成功')
        except Exception as e:
            save_log(request, Module.DATA, OperationType.ADD, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='update_company', description='修改公司信息')
    @ns.expect(company_data)
    def put(self):
        """修改公司基本信息"""

        company_id = request.form.get('company_id')
        management_unit = request.form.get('management_unit')
        custodian_unit = request.form.get('custodian_unit')
        introduction = request.form.get('introduction')
        contact_person = request.form.get('contact_person')
        contact_information = request.form.get('contact_information')
        pictures = request.files.getlist('picture')

        if company_id is None:
            return jsonify(code=StatesCode.PARA_ERROR, message='公司id不能为空')

        pictures_base64 = []

        if pictures:
            for picture in pictures:

                ext = picture.filename.split('.')[-1]
                base64_data = b'data:image/%s;base64,' % ext.encode('utf-8') + base64.b64encode(picture.read())
                base64_data = base64_data.decode('utf-8')
                pictures_base64.append(base64_data)

        try:
            with Session(current_app.engine) as session:
                stmt = update(CompanyData).where(CompanyData.id == company_id).values(
                    management_unit=management_unit,
                    custodian_unit=custodian_unit,
                    introduction=introduction,
                    contact_person=contact_person,
                    contact_information=contact_information,
                    picture=json.dumps(pictures_base64),
                )

                session.execute(stmt)
                session.commit()

            save_log(request, Module.DATA, OperationType.UPDATE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='修改成功')
        except Exception as e:
            save_log(request, Module.DATA, OperationType.UPDATE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


@ns.route('/building_list')
class BuildingListApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='building_list', description='获取楼宇列表')
    def get(self):
        """获取楼宇列表"""
        try:
            with Session(current_app.engine) as session:
                stmt = select(Building)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='获取成功', data=to_dict(results))
        except Exception as e:
            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


building = reqparse.RequestParser(bundle_errors=True)
building.add_argument(name='building_id', type=int, location='args', required=False, help='楼宇id')

building_details = reqparse.RequestParser(bundle_errors=True)
building_details.add_argument(name='building_id', type=int, location='form', required=False, help='楼宇id')
building_details.add_argument(name='building_name', type=str, location='form', required=False, help='楼宇名')
building_details.add_argument(name='building_area', type=str, location='form', required=False, help='楼宇面积')
building_details.add_argument(name='building_floor', type=str, location='form', required=False, help='建筑楼层')
building_details.add_argument(name='trem', type=str, location='form', required=False, help='建筑年限')
building_details.add_argument(name='date', type=str, location='form', required=False, help='建成日期')
building_details.add_argument(name='contact_person', type=str, location='form', required=False, help='联系人')
building_details.add_argument(name='contact_information', type=str, location='form', required=False, help='联系人方式')
building_details.add_argument(name='building_introduction', type=str, location='form', required=False, help='楼宇简介')
building_details.add_argument(name='picture', type=FileStorage, location='files', required=False, help='楼宇图片')


@ns.route('/building')
class BuildingApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='building_list', description='获取楼宇列表')
    @ns.expect(building)
    def get(self):
        """获取楼宇信息"""
        building_id = request.args.get('building_id')

        if building_id is None:
            return jsonify(code=StatesCode.PARA_ERROR, message='楼宇id不能为空')

        try:
            with Session(current_app.engine) as session:
                stmt = select(Building).where(Building.id == building_id)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='获取成功', data=to_dict(results))
        except Exception as e:
            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='add_building', description='增加楼宇')
    @ns.expect(building_details)
    def post(self):
        """添加楼宇"""
        building_name = request.form.get('building_name')
        building_area = request.form.get('building_area')
        building_floor = request.form.get('building_floor')
        trem = request.form.get('trem')
        date = request.form.get('date')
        contact_person = request.form.get('contact_person')
        contact_information = request.form.get('contact_information')
        building_introduction = request.form.get('building_introduction')
        pictures = request.files.getlist('picture')

        if building_name is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message='楼宇名称不能为空')

        pictures_base64 = []
        if pictures:
            for picture in pictures:

                ext = picture.filename.split('.')[-1]
                base64_data = b'data:image/%s;base64,' % ext.encode('utf-8') + base64.b64encode(picture.read())
                base64_data = base64_data.decode('utf-8')
                pictures_base64.append(base64_data)

        try:
            with Session(current_app.engine) as session:
                stmt = insert(Building).values(
                    name=building_name,
                    area=building_area,
                    floor=building_floor,
                    trem=trem,
                    date=date,
                    contact_person=contact_person,
                    contact_information=contact_information,
                    introduction=building_introduction,
                    picture=json.dumps(pictures_base64)
                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.DATA, OperationType.ADD, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='添加成功')
        except Exception as e:
            save_log(request, Module.DATA, OperationType.ADD, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='update_building', description='修改楼宇')
    @ns.expect(building_details)
    def put(self):
        """修改楼宇"""
        building_id = request.form.get('building_id')
        building_name = request.form.get('building_name')
        building_area = request.form.get('building_area')
        building_floor = request.form.get('building_floor')
        trem = request.form.get('trem')
        date = request.form.get('date')
        contact_person = request.form.get('contact_person')
        contact_information = request.form.get('contact_information')
        building_introduction = request.form.get('building_introduction')
        pictures = request.files.getlist('picture')

        if building_id is None and building_name is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message='楼宇id和楼宇名称不能为空')

        pictures_base64 = []
        if pictures:
            for picture in pictures:

                ext = picture.filename.split('.')[-1]
                base64_data = b'data:image/%s;base64,' % ext.encode('utf-8') + base64.b64encode(picture.read())
                base64_data = base64_data.decode('utf-8')
                pictures_base64.append(base64_data)

        try:
            with Session(current_app.engine) as session:
                stmt = update(Building).where(Building.id == building_id).values(
                    name=building_name,
                    area=building_area,
                    floor=building_floor,
                    trem=trem,
                    date=date,
                    contact_person=contact_person,
                    contact_information=contact_information,
                    introduction=building_introduction,
                    picture=json.dumps(pictures_base64)
                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.DATA, OperationType.UPDATE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='修改成功')
        except Exception as e:
            save_log(request, Module.DATA, OperationType.UPDATE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='delete_building', description='删除楼宇')
    @ns.expect(building_details)
    def delete(self):
        """删除楼宇"""
        building_id = request.form.get('building_id')
        if building_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message='楼宇id不能为空')

        try:
            with Session(current_app.engine) as session:
                stmt = delete(Building).where(Building.id == building_id)
                session.execute(stmt)
                session.commit()

            save_log(request, Module.DATA, OperationType.DELETE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='删除成功')
        except Exception as e:
            save_log(request, Module.DATA, OperationType.DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


underlying_system = reqparse.RequestParser(bundle_errors=True)
underlying_system.add_argument(name='picture', type=FileStorage, location='files', required=False, help='楼宇图片')


@ns.route('/underlying_system')
class UnderlyingSystemMessageApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='underlying_system_list', description='获取底层系统')
    @ns.expect()
    def get(self):
        """获取底层系统"""

        try:
            with Session(current_app.engine) as session:
                stmt = select(UnderlyingSystem)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=to_dict(results))
        except Exception as e:
            save_log(request, Module.DATA, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='update_underlying_system', description='添加底层系统')
    @ns.expect(underlying_system)
    def post(self):
        """添加底层系统信息"""
        picture = request.files.get('picture')

        if picture:
            ext = picture.filename.split('.')[-1]
            base64_data = b'data:image/%s;base64,' % ext.encode('utf-8') + base64.b64encode(picture.read())
            base64_data = base64_data.decode('utf-8')
        else:
            base64_data = None

        try:
            with Session(current_app.engine) as session:
                stmt = insert(UnderlyingSystem).values(
                    picture=base64_data
                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.DATA, OperationType.ADD, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功')

        except Exception as e:
            save_log(request, Module.DATA, OperationType.ADD, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))
