import base64

from flask import request, jsonify, current_app, g
from flask_restx import Resource, Namespace, reqparse
from sqlalchemy import select, insert, update, delete
from sqlalchemy.orm import Session
from werkzeug.datastructures import FileStorage

from app.defines import Module, OperationType, StatesCode
from app.modle.information import FloorConfiguration, StaffConfiguration, SecurityPerson
from app.utils.jwt_util import login_required
from app.utils.save_log import save_log
from app.utils.util import to_dict
from config import Config

ns = Namespace('information', description='信息管理接口')

config = Config()


def information_list(rows):
    res = []
    for row in rows:
        res.append({"id": row.id, "name": row.name})

    return res


@ns.route('/information_list')
class InformationTypeApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='information_list', description='信息列表')
    def get(self):
        """获取信息列表"""
        information_type = config.common.INFORMATION_TYPE

        data = {}
        try:
            with Session(current_app.engine) as session:

                for information in information_type:
                    if information == '楼层地图信息配置':

                        stmt = select(FloorConfiguration.id, FloorConfiguration.name)

                        data[information] = information_list(session.execute(stmt))

                    elif information == '安保人员信息配置':

                        stmt = select(StaffConfiguration.id, StaffConfiguration.name)

                        data[information] = information_list(session.execute(stmt))

            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=data)

        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


floor_configuration_details = reqparse.RequestParser(bundle_errors=True)
floor_configuration_details.add_argument(name='id', type=str, location='args', required=False, help='配置id')

floor_configuration = reqparse.RequestParser(bundle_errors=True)
floor_configuration.add_argument(name='id', type=str, location='form', required=False, help='配置id')
floor_configuration.add_argument(name='name', type=str, location='form', required=False, help='配置名称')
floor_configuration.add_argument(name='location', type=str, location='form', required=False, help='楼层位置')
floor_configuration.add_argument(name='use', type=str, location='form', required=False, help='楼层用途')
floor_configuration.add_argument(name='remark', type=str, location='form', required=False, help='备注')
floor_configuration.add_argument(name='attachment', type=FileStorage, location='files', required=False, help='附件')


@ns.route('/floor_configuration')
class FloorConfigurationApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='floor_configuration', description='获取楼层地图信息配置')
    @ns.expect(floor_configuration_details)
    def get(self):
        """获取楼层地图信息配置"""

        floor_configuration_id = request.args.get('id')

        try:
            with Session(current_app.engine) as session:
                stmt = select(FloorConfiguration).where(FloorConfiguration.id == floor_configuration_id)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=to_dict(results))
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='add_floor_configuration', description='添加楼层地图信息配置')
    @ns.expect(floor_configuration)
    def post(self):
        """添加楼层地图信息配置"""
        name = request.form.get('name')
        location = request.form.get('location')
        use = request.form.get('use')
        remark = request.form.get('remark')
        attachment = request.files.get('attachment')

        if name is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置名不能为空")

        if attachment:
            ext = attachment.filename.split('.')[-1]
            base64_data = b'data:image/%s;base64,' % ext.encode('utf-8') + base64.b64encode(attachment.read())
            base64_data = base64_data.decode('utf-8')
        else:
            base64_data = None

        try:
            with Session(current_app.engine) as session:
                stmt = insert(FloorConfiguration).values(
                    name=name,
                    location=location,
                    use=use,
                    remark=remark,
                    attachment=base64_data,
                    account=g.user_name
                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.INFORMATION, OperationType.ADD, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='添加成功')
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.ADD, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='update_floor_configuration', description='修改楼层地图信息配置')
    @ns.expect(floor_configuration)
    def put(self):
        """修改楼层地图信息配置"""

        floor_configuration_id = request.form.get('id')
        name = request.form.get('name')
        location = request.form.get('location')
        use = request.form.get('use')
        remark = request.form.get('remark')
        attachment = request.files.get('attachment')

        if name is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置名不能为空")

        if attachment:
            ext = attachment.filename.split('.')[-1]
            base64_data = b'data:image/%s;base64,' % ext.encode('utf-8') + base64.b64encode(attachment.read())
            base64_data = base64_data.decode('utf-8')
        else:
            base64_data = None

        try:
            with Session(current_app.engine) as session:
                stmt = update(FloorConfiguration).where(FloorConfiguration.id == floor_configuration_id).values(
                    name=name,
                    location=location,
                    use=use,
                    remark=remark,
                    attachment=base64_data,
                    account=g.user_name
                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.INFORMATION, OperationType.UPDATE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='修改成功')
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.UPDATE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='delete_floor_configuration', description='删除楼层地图信息配置')
    @ns.expect(floor_configuration)
    def delete(self):
        """删除楼层地图信息配置"""
        floor_configuration_id = request.form.get('id')

        if floor_configuration_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = delete(FloorConfiguration).where(FloorConfiguration.id == floor_configuration_id)
                session.execute(stmt)
                session.commit()

            save_log(request, Module.INFORMATION, OperationType.DELETE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message="删除成功")
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


@ns.route('/security_person')
class GetSecurityPersonApi(Resource):
    method_decorators = [login_required]

    def get(self):
        """获取人员名单"""
        try:
            with Session(current_app.engine) as session:
                stmt = select(SecurityPerson)
                results = information_list(session.execute(stmt))

            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=results)
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


staff_configuration_details = reqparse.RequestParser(bundle_errors=True)
staff_configuration_details.add_argument(name='id', type=str, location='args', required=False, help='配置id')

floor_configuration = reqparse.RequestParser(bundle_errors=True)
floor_configuration.add_argument(name='id', type=str, location='form', required=False, help='配置id')
floor_configuration.add_argument(name='name', type=str, location='form', required=False, help='配置名称')
floor_configuration.add_argument(name='personnel_type', type=str, location='form', required=False, help='人员类别')
floor_configuration.add_argument(name='responsibility_range', type=str, location='form', required=False,
                                 help='职责范围')
floor_configuration.add_argument(name='staff_list', type=str, location='form', required=False, help='人员名单')
floor_configuration.add_argument(name='remark', type=str, location='form', required=False, help='备注')


@ns.route('/staff_configuration')
class StaffConfigurationApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='staff_configuration', description='获取人员配置信息')
    @ns.expect(staff_configuration_details)
    def get(self):
        """获取安保人员信息配置"""

        staff_configuration_id = request.args.get('id')
        if staff_configuration_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = select(StaffConfiguration).where(StaffConfiguration.id == staff_configuration_id)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=to_dict(results))
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='add_staff_configuration', description='添加人员配置信息')
    @ns.expect(floor_configuration)
    def post(self):
        """添加安保人员信息配置"""
        name = request.form.get('name')
        personnel_type = request.form.get('personnel_type')
        responsibility_range = request.form.get('responsibility_range')
        staff_list = request.form.get('staff_list')
        remark = request.form.get('remark')

        try:
            with Session(current_app.engine) as session:
                stmt = insert(StaffConfiguration).values(
                    name=name,
                    personnel_type=personnel_type,
                    responsibility_range=responsibility_range,
                    staff_list=staff_list,
                    remark=remark
                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.INFORMATION, OperationType.ADD, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功')

        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.ADD, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='update_staff_configuration', description='修改人员配置信息')
    @ns.expect(floor_configuration)
    def put(self):
        """修改安保人员信息配置"""
        staff_configuration_id = request.form.get('id')
        name = request.form.get('name')
        personnel_type = request.form.get('personnel_type')
        responsibility_range = request.form.get('responsibility_range')
        staff_list = request.form.get('staff_list')
        remark = request.form.get('remark')

        if staff_configuration_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = update(StaffConfiguration).where(StaffConfiguration.id == staff_configuration_id).values(
                    name=name,
                    personnel_type=personnel_type,
                    responsibility_range=responsibility_range,
                    staff_list=staff_list,
                    remark=remark
                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.INFORMATION, OperationType.UPDATE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功')

        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.UPDATE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='delete_staff_configuration', description='删除人员配置信息')
    @ns.expect(floor_configuration)
    def delete(self):
        """删除安保人员信息配置"""

        staff_configuration_id = request.form.get('id')

        if staff_configuration_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = delete(StaffConfiguration).where(StaffConfiguration.id == staff_configuration_id)
                session.execute(stmt)
                session.commit()

            save_log(request, Module.INFORMATION, OperationType.DELETE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功')

        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))
